==> cf-variable-resolver-0.0.8.tgz: package/lib/__tests__/vault.unit.spec.js <==
/* eslint-disable no-undef */
/* eslint-disable no-template-curly-in-string */
jest.mock('node-vault');
const CFError = require('cf-errors');
const Vault = require('../clients/hashicorp-vault');
const nodeVault = require('node-vault');

const apiUrl = 'http://some.vault.url';

function createVault(auth) {
    const mockClient = {
        tokenLookupSelf: jest.fn(),
        approleLogin: jest.fn(),
        userpassLogin: jest.fn(),
        read: jest.fn(),
    };
    nodeVault.mockImplementationOnce(() => mockClient);
    return [mockClient, new Vault(apiUrl, auth)];
}

describe('Vault', () => {
    describe('constructor', () => {
        it('Should throw if no apiUrl is supplied', () => {
            expect(() => new Vault(undefined, {})).toThrow(new CFError('Failed to build client'));
        });

        it('Should throw if no auth is supplied', () => {
            expect(() => new Vault(apiUrl, {})).toThrow(new CFError('Failed to authenticate to the specified secret-store'));
        });

        it('Should create an instance when arguments are supplied', () => {
            const vault = new Vault(apiUrl, { type: 'some_type' });
            expect(vault).toBeTruthy();
        });
    });

    describe('init', () => {
        describe('token', () => {
            const type = 'token';

            it('Should throw if no token is supplied', async () => {
                const [mockClient, vault] = createVault({ type, token: undefined });
                mockClient.tokenLookupSelf.mockImplementationOnce(async () => { throw new Error('Missing token'); });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. Missing token'));
                expect(mockClient.tokenLookupSelf.mock.calls.length).toBe(1);
            });

            it('Should successfully get token data', async () => {
                const token = 'some_token';
                const [mockClient, vault] = createVault({ type, token });
                mockClient.tokenLookupSelf.mockImplementationOnce(async () => ({}));
                await vault.init();
                expect(mockClient.tokenLookupSelf.mock.calls.length).toBe(1);
            });
        });

        describe('userpass', () => {
            const type = 'userpass';
            const username = 'some_ser';
            const password = 'some_pssword';

            it('Should throw if no username is supplied', async () => {
                const [mockClient, vault] = createVault({ type, username: undefined, password: undefined });
                mockClient.userpassLogin.mockImplementationOnce(async () => { throw new Error('Missing username'); });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. Missing username'));
                expect(mockClient.userpassLogin.mock.calls.length).toBe(1);
                expect(mockClient.userpassLogin.mock.calls[0]).toEqual([{ type, username: undefined, password: undefined }]);
            });

            it('Should throw if no password is supplied', async () => {
                const [mockClient, vault] = createVault({ type, username, password: undefined });
                mockClient.userpassLogin.mockImplementationOnce(async () => { throw new Error('Missing password'); });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. Missing password'));
                expect(mockClient.userpassLogin.mock.calls.length).toBe(1);
                expect(mockClient.userpassLogin.mock.calls[0]).toEqual([{ type, username, password: undefined }]);
            });

            it('Should Fail with wrong credentials', async () => {
                const [mockClient, vault] = createVault({ type, username, password });
                mockClient.userpassLogin.mockImplementationOnce(async () => { throw new Error('invalid username or password'); });
                await expect(vault.init()).rejects.toThrowError(new CFError(
                    'Failed to authenticate to the specified secret-store. invalid username or password',
                ));
                expect(mockClient.userpassLogin.mock.calls.length).toBe(1);
                expect(mockClient.userpassLogin.mock.calls[0]).toEqual([{ type, username, password }]);
            });

            it('Should successfully get token data', async () => {
                const [mockClient, vault] = createVault({ type, username, password });
                mockClient.userpassLogin.mockImplementationOnce(async () => ({}));
                await vault.init();
                expect(mockClient.userpassLogin.mock.calls.length).toBe(1);
                expect(mockClient.userpassLogin.mock.calls[0]).toEqual([{ type, username, password }]);
            });
        });

        describe('approle', () => {
            const type = 'approle';
            const roleId = 'some_role';
            const secretId = 'some_secret';

            it('Should throw if no roleId is supplied', async () => {
                const [mockClient, vault] = createVault({ type, roleId: undefined, secretId: undefined });
                mockClient.approleLogin.mockImplementationOnce(async () => { throw new Error('Missing roleId'); });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. Missing roleId'));
                expect(mockClient.approleLogin.mock.calls.length).toBe(1);
                expect(mockClient.approleLogin.mock.calls[0]).toEqual([{ type, roleId: undefined, secretId: undefined }]);
            });

            it('Should throw if no secretId is supplied', async () => {
                const [mockClient, vault] = createVault({ type, roleId, secretId: undefined });
                mockClient.approleLogin.mockImplementationOnce(async () => { throw new Error('Missing secretId'); });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. Missing secretId'));
                expect(mockClient.approleLogin.mock.calls.length).toBe(1);
                expect(mockClient.approleLogin.mock.calls[0]).toEqual([{ type, roleId, secretId: undefined }]);
            });

            it('Should Fail with wrong roleId', async () => {
                const [mockClient, vault] = createVault({ type, roleId, secretId });
                mockClient.approleLogin.mockImplementationOnce(async () => { throw new Error('Invalid roleId'); });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. Invalid roleId'));
                expect(mockClient.approleLogin.mock.calls.length).toBe(1);
                expect(mockClient.approleLogin.mock.calls[0]).toEqual([{ type, roleId, secretId }]);
            });

            it('Should Fail with wrong secredId', async () => {
                const [mockClient, vault] = createVault({ type, roleId, secretId });
                mockClient.approleLogin.mockImplementationOnce(async () => { throw new Error('Invalid secretId'); });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. Invalid secretId'));
                expect(mockClient.approleLogin.mock.calls.length).toBe(1);
                expect(mockClient.approleLogin.mock.calls[0]).toEqual([{ type, roleId, secretId }]);
            });

            it('Should successfully get token data', async () => {
                const [mockClient, vault] = createVault({ type, roleId, secretId });
                mockClient.approleLogin.mockImplementationOnce(async () => ({}));
                await vault.init();
                expect(mockClient.approleLogin.mock.calls.length).toBe(1);
                expect(mockClient.approleLogin.mock.calls[0]).toEqual([{ type, roleId, secretId }]);
            });
        });
    });
});

==> cf-variable-resolver-0.0.16.tgz: package/lib/__tests__/vault.unit.spec.js <==
/* eslint-disable no-undef */
/* eslint-disable no-template-curly-in-string */
const fs = require('fs').promises;
const { URL } = require('url');
const CFError = require('cf-errors');
const nodeVault = require('node-vault');
const Vault = require('../clients/hashicorp-vault');

const apiUrl = 'http://some.vault.url';

function createVault(auth) {
    const mockClient = {
        tokenLookupSelf: jest.fn(),
        approleLogin: jest.fn(),
        userpassLogin: jest.fn(),
        gcpLogin: jest.fn(),
        kubernetesLogin: jest.fn(),
        read: jest.fn(),
    };
    nodeVault.mockImplementationOnce(() => mockClient);
    return [mockClient, new Vault(apiUrl, auth)];
}

describe('Vault', () => {
    describe('constructor', () => {
        it('Should throw if no apiUrl is supplied', () => {
            expect(() => new Vault(undefined, {})).toThrow(new CFError('Failed to build client'));
        });

        it('Should throw if no auth is supplied', () => {
            expect(() => new Vault(apiUrl, {})).toThrow(new CFError('Failed to authenticate to the specified secret-store'));
        });

        it('Should create an instance when arguments are supplied', () => {
            const vault = new Vault(apiUrl, { type: 'some_type' });
            expect(vault).toBeTruthy();
        });
    });

    describe('init', () => {
        describe('token', () => {
            const type = 'token';

            it('Should throw if no token is supplied', async () => {
                const [mockClient, vault] = createVault({ type, token: undefined });
                mockClient.tokenLookupSelf.mockImplementationOnce(async () => { throw new Error('Missing token'); });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. Missing token'));
                expect(mockClient.tokenLookupSelf.mock.calls).toHaveLength(1);
            });

            it('Should successfully get token data', async () => {
                const token = 'some_token';
                const [mockClient, vault] = createVault({ type, token });
                mockClient.tokenLookupSelf.mockImplementationOnce(async () => ({}));
                await vault.init();
                expect(mockClient.tokenLookupSelf.mock.calls).toHaveLength(1);
            });
        });

        describe('userpass', () => {
            const type = 'userpass';
            const username = 'some_user';
            const password = 'some_pssword';

            it('Should throw if no username is supplied', async () => {
                const [mockClient, vault] = createVault({ type, username: undefined, password: undefined });
                mockClient.userpassLogin.mockImplementationOnce(async () => { throw new Error('Missing username'); });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. Missing username'));
                expect(mockClient.userpassLogin.mock.calls).toHaveLength(1);
                expect(mockClient.userpassLogin.mock.calls[0]).toEqual([{ type, username: undefined, password: undefined }]);
            });

            it('Should throw if no password is supplied', async () => {
                const [mockClient, vault] = createVault({ type, username, password: undefined });
                mockClient.userpassLogin.mockImplementationOnce(async () => { throw new Error('Missing password'); });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. Missing password'));
                expect(mockClient.userpassLogin.mock.calls).toHaveLength(1);
                expect(mockClient.userpassLogin.mock.calls[0]).toEqual([{ type, username, password: undefined }]);
            });

            it('Should Fail with wrong credentials', async () => {
                const [mockClient, vault] = createVault({ type, username, password });
                mockClient.userpassLogin.mockImplementationOnce(async () => { throw new Error('invalid username or password'); });
                await expect(vault.init()).rejects.toThrowError(new CFError(
                    'Failed to authenticate to the specified secret-store. invalid username or password',
                ));
                expect(mockClient.userpassLogin.mock.calls).toHaveLength(1);
                expect(mockClient.userpassLogin.mock.calls[0]).toEqual([{ type, username, password }]);
            });

            it('Should successfully get token data', async () => {
                const [mockClient, vault] = createVault({ type, username, password });
                mockClient.userpassLogin.mockImplementationOnce(async () => ({}));
                await vault.init();
                expect(mockClient.userpassLogin.mock.calls).toHaveLength(1);
                expect(mockClient.userpassLogin.mock.calls[0]).toEqual([{ type, username, password }]);
            });
        });

        describe('approle', () => {
            const type = 'approle';
            const roleId = 'some_role';
            const secretId = 'some_secret';

            it('Should throw if no roleId is supplied', async () => {
                const [mockClient, vault] = createVault({ type, roleId: undefined, secretId: undefined });
                mockClient.approleLogin.mockImplementationOnce(async () => { throw new Error('Missing roleId'); });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. Missing roleId'));
                expect(mockClient.approleLogin.mock.calls).toHaveLength(1);
                expect(mockClient.approleLogin.mock.calls[0]).toEqual([{ type, roleId: undefined, secretId: undefined }]);
            });

            it('Should throw if no secretId is supplied', async () => {
                const [mockClient, vault] = createVault({ type, roleId, secretId: undefined });
                mockClient.approleLogin.mockImplementationOnce(async () => { throw new Error('Missing secretId'); });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. Missing secretId'));
                expect(mockClient.approleLogin.mock.calls).toHaveLength(1);
                expect(mockClient.approleLogin.mock.calls[0]).toEqual([{ type, roleId, secretId: undefined }]);
            });

            it('Should Fail with wrong roleId', async () => {
                const [mockClient, vault] = createVault({ type, roleId, secretId });
                mockClient.approleLogin.mockImplementationOnce(async () => { throw new Error('Invalid roleId'); });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. Invalid roleId'));
                expect(mockClient.approleLogin.mock.calls).toHaveLength(1);
                expect(mockClient.approleLogin.mock.calls[0]).toEqual([{ type, roleId, secretId }]);
            });

            it('Should Fail with wrong secredId', async () => {
                const [mockClient, vault] = createVault({ type, roleId, secretId });
                mockClient.approleLogin.mockImplementationOnce(async () => { throw new Error('Invalid secretId'); });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. Invalid secretId'));
                expect(mockClient.approleLogin.mock.calls).toHaveLength(1);
                expect(mockClient.approleLogin.mock.calls[0]).toEqual([{ type, roleId, secretId }]);
            });

            it('Should successfully get token data', async () => {
                const [mockClient, vault] = createVault({ type, roleId, secretId });
                mockClient.approleLogin.mockImplementationOnce(async () => ({}));
                await vault.init();
                expect(mockClient.approleLogin.mock.calls).toHaveLength(1);
                expect(mockClient.approleLogin.mock.calls[0]).toEqual([{ type, roleId, secretId }]);
            });
        });

        describe('gcp', () => {
            const type = 'gcp';
            const role = 'some_role';

            it('Should throw if roleType is not supplied', async () => {
                const [, vault] = createVault({ type, roleType: undefined, roleId: undefined });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. GCE Role Type \'undefined\' is not supported'));
            });

            it('Should throw if roleType is not "iam"', async () => {
                const [, vault] = createVault({ type, roleType: 'iam', roleId: undefined });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. GCE Role Type \'iam\' is not supported'));
            });

            it('Should get a token from GCE', async () => {
                const jwt = 'some_gce_token';
                const [mockClient, vault] = createVault({ type, roleType: 'gce', role });
                fetch.mockResponse(async (req) => {
                    const [baseUrl] = req.url.split('?');
                    const url = new URL(req.url);
                    const searchParams = Object.fromEntries(url.searchParams);
                    expect(baseUrl).toBe(`http://metadata/computeMetadata/v1/instance/service-accounts/default/identity`);
                    expect(searchParams).toEqual({
                        audience: `http://vault/${role}`,
                        format: 'full',
                    });
                    return jwt;
                });
                mockClient.gcpLogin.mockImplementationOnce(async ({ role: actualRole, jwt: actualJwt }) => {
                    expect(actualRole).toBe(role);
                    expect(actualJwt).toBe(jwt);
                });
                await vault.init();
                expect(mockClient.gcpLogin.mock.calls).toHaveLength(1);
            });
        });

        describe('k8s', () => {
            const type = 'kubernetes';
            const role = 'some_role';

            it('Should throw if role is not supplied', async () => {
                const [mockClient, vault] = createVault({ type, role: undefined });
                mockClient.kubernetesLogin.mockImplementationOnce(async () => { throw new Error('Missing role'); });
                await expect(vault.init()).rejects.toThrowError(new CFError('Failed to authenticate to the specified secret-store. Missing role'));
            });

            it('Should use jwt token from auth, if available', async () => {
                const jwt = 'some_gce_token';
                const [mockClient, vault] = createVault({ type, role, jwt });
                mockClient.kubernetesLogin.mockImplementationOnce(async ({ role: actualRole, jwt: actualJwt }) => {
                    expect(actualRole).toBe(role);
                    expect(actualJwt).toBe(jwt);
                });
                await vault.init();
                expect(fs.readFile.mock.calls).toHaveLength(0);
                expect(mockClient.kubernetesLogin.mock.calls).toHaveLength(1);
            });

            it('Should use jwt token from file, if none was supplied', async () => {
                const jwt = 'some_k8s_token';
                const [mockClient, vault] = createVault({ type, role });
                fs.readFile.mockImplementationOnce(async (path) => {
                    expect(path).toBe('/var/run/secrets/kubernetes.io/serviceaccount/token');
                    return jwt;
                });
                mockClient.kubernetesLogin.mockImplementationOnce(async ({ role: actualRole, jwt: actualJwt }) => {
                    expect(actualRole).toBe(role);
                    expect(actualJwt).toBe(jwt);
                });
                await vault.init();
                expect(fs.readFile.mock.calls).toHaveLength(1);
                expect(mockClient.kubernetesLogin.mock.calls).toHaveLength(1);
            });
        });
    });
});
